//--------------------------------------------------------------------------------------------------
//
// Title       : vga
// Design      : VGA
// Author      : 0
// Company     : 0
//
//-------------------------------------------------------------------------------------------------
//
// File        : vga.v
// Generated   : Sun Jan  4 16:21:10 2004
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps

module VGA_DISP (clk, reset, hcnt, vcnt, hsyncb, vsyncb, en, addr) ;

input clk ;
input reset ;
output [9:0] hcnt ;
output [9:0] vcnt ;
output hsyncb ;
output vsyncb ;
input en ;
output [14:0] addr;

wire clk;
wire reset;
wire en;

reg hsyncb;
reg vsyncb;
reg [9:0] hcnt;
reg [9:0] vcnt;
reg [14:0] addr;

`define SCAN_LINE_TIME_MAX_TIC 793  // 31.77us * 25 MHz = 794.2 
`define LINE_NUM_LIM 527            // 16.784 m / 31.77u = 528.29
`define ACTIVE_VIDEO_TIC 629        // 25.17us * 25 MHz = 629.25 (Screen Size for Horizon)
`define FRONT_PORCH_TIC 24          // 0.94 us * 25MHz = 23.5
`define HRZ_SYNC_TIC 94             // 3.77 us * 25 MHz = 94.25 
`define BACK_PORCH_TIC 47           // 1.89 us * 25 MHz = 47.25
`define ACTIVE_VERT_LINE 480        // 15.25ms/31.77us = 480.01 (Screen Size for Vertical)
`define FRONT_PORCH_LINE 11
`define VERT_SYNC_LINE 2
`define BACK_PORCH_LINE 22

wire detect_hsyncb;
reg buf_hsyncb;

wire [14:0] addr2;
assign addr2 = ((hcnt[4:0] == 5'b11111) ?
  ({7'b0000000,vcnt[7:4],hcnt[8:5]} + 1) : ({7'b0000000,vcnt[7:4],hcnt[8:5]}));

always @(posedge clk or negedge reset) begin
  if(~reset) begin
    hcnt <= 0;
    addr <= 0;
  end
  else begin
    if((hcnt < `SCAN_LINE_TIME_MAX_TIC)&&(en == 1'b1)) begin
      hcnt <= hcnt + 1;
    end
    else begin
      hcnt <= 0;
    end
    addr <= addr2;
  end
end

always @(posedge clk or negedge reset) begin
  if(~reset)
    buf_hsyncb <= 1'b1;
  else
    buf_hsyncb <= hsyncb;
end

assign detect_hsyncb = (hsyncb) & (~buf_hsyncb);

always @(posedge clk or negedge reset) begin
  if(~reset)
    vcnt <= 0;
  else if(detect_hsyncb) begin
    if((vcnt < `LINE_NUM_LIM)&&(en == 1'b1))
      vcnt <= vcnt + 1;
    else
      vcnt <= 0;
  end
end

//for centering the horizon position hsyncb should begin asserted at the time
// + 600 <= Real display period
// + (629 - 600)/2 <= the pixel that is not used from the display
// + 24 <= FRONT_PORCH_TIC
// = 600 + 15 + 24 = 639 
always @(posedge clk or negedge reset) begin
  if(~reset)
    hsyncb <= 1'b1;
  else begin
    if(((hcnt >= 595)&&(hcnt < 595 + `HRZ_SYNC_TIC))||(~en))
      hsyncb <= 1'b0;
    else
      hsyncb <= 1'b1;
  end
end

//for centering the vertical position vsyncb should begin asserted at the time
// +  450 <= Real display period
// + (480 - 450)/2 <= the pixel that is not used from the display
// + 11 <= FRONT_PORCH_TIC
// = 450 + 15 + 11 = 476 line
always @(posedge clk or negedge reset) begin
  if(~reset)
    vsyncb <= 1'b1;
  else begin
    if(((vcnt >= 476-1)&&(vcnt < 476 + `VERT_SYNC_LINE - 1))||(~en))
      vsyncb <= 0;
    else
      vsyncb <= 1;
  end
end
endmodule
